@@ -8,7 +8,7 @@ use warnings;
 use base qw(Class::Accessor);
 use Carp qw(carp croak);
 
-use version; our $VERSION = qw(1.0.1);
+use version; our $VERSION = qw(1.1.0);
 
 
 #-------------------------------------------------------------------
@@ -17,8 +17,20 @@ use version; our $VERSION = qw(1.0.1);
 
 #By default, use the sacctmgr found in your path
 my $DEFAULT_SACCTMGR_CMD='sacctmgr';
-#If you wish to default to something else, uncomment and change the line below
+#If you wish to default to something else, you can either modify the
+#class method default_sacctmgr_path_version, and/or uncomment 
+#and change the line below. 
 #$DEFAULT_SACCTMGR_CMD='/usr/local/slurm/bin/sacctmgr';
+#It is also recommended that you set the $DEFAULT_SACCTMGR_VERSION to the
+#Slurm version number associated with that specified sacctmgr command.
+#Failure to do so might result in the package needing to issue extra 
+#sacctmgr commands to determine what version of sacctmgr being run, which is inefficient
+my $DEFAULT_SACCTMGR_VERSION;
+#Uncomment and modify this if the Slurm version for the sacctmgr command listed
+#in $DEFAULT_SACCTMGR_CMD is known
+#$DEFAULT_SACCTMGR_VERSION='15.08.2';
+#The default version of default_sacctmgr_path_version just returns these
+#two values; modified versions might or might not use these values.
 
 #This is intended for regression tests only
 my $_last_raw_output;
@@ -26,6 +38,13 @@ sub _sacctmgr_last_raw_output($)
 {       return $_last_raw_output;
 }
 
+my @SACCTMGR_CMD_CAPABILITIES_LIST =
+(	'trackable_resources', #Supports TRES, at least at basic level
+);
+
+my %SACCTMGR_CMD_CAPABILITIES_HASH = map { $_ => undef } @SACCTMGR_CMD_CAPABILITIES_LIST;
+
+
 #-------------------------------------------------------------------
 #	Accessors
 #-------------------------------------------------------------------
@@ -37,7 +56,10 @@ my @rw_accessors = qw(
 
 my @ro_accessors = qw(
 	sacctmgr
+	slurm_version
 );
+#	_sacctmgr_cmd_capabilities
+#	_cached_sacctmgr_cmd_capabilities
 
 __PACKAGE__->mk_accessors(@rw_accessors);
 __PACKAGE__->mk_ro_accessors(@ro_accessors);
@@ -47,6 +69,24 @@ my @required_parms = qw(
 	sacctmgr
 );
 
+#-------	Special accessors/mutators
+
+sub slurm_version($;$)
+#Gets version of slurm/sacctmgr cmd
+#Preferably our cached version, but (unless $cachedonly flag is set)
+#will call sacctmgr to get it if needed
+#If unknown and $cachedonly set, will return undef.
+{	my $self = shift;
+	my $cachedonly = shift;
+
+	my $svers = $self->get('slurm_version');
+	return $svers if $svers;
+
+	return if $cachedonly;
+
+	return $self->_determine_slurm_version; #This sets 'slurm_version' data member
+}
+
 #-------------------------------------------------------------------
 #	Constructors, etc
 #-------------------------------------------------------------------
@@ -82,6 +122,23 @@ sub _parse_args($@)
 	{	next ROARG unless exists $args{$arg};
 		$val = delete $args{$arg};
 		next ROARG unless defined $val;
+
+		if ( $arg eq 'sacctmgr' || $arg eq 'slurm_version' )
+		{	#This gets handled specially
+			my ($tmppath, $tmpver);
+			if ( $arg eq 'sacctmgr' )
+			{	$tmppath = $val;
+				$tmpver = delete $args{'slurm_version'};
+			} elsif ( $arg eq 'slurm_version' )
+			{	$tmpver = $val;
+				$tmppath = delete $args{'sacctmgr'};
+			} else
+			{	die "Should not reach here at ";
+			}
+			$obj->_set_sacctmgr_path($tmppath, $tmpver);
+			next ROARG;
+		}
+
 		$meth = $arg;
 		$obj->set($meth,$val);
 	}
@@ -99,7 +156,10 @@ sub _set_defaults($)
 
 	my ($tmp);
 	$tmp = $obj->sacctmgr;
-	$obj->set('sacctmgr', $DEFAULT_SACCTMGR_CMD) unless $tmp; 
+	unless ( $tmp )
+	{	my @tmp = $obj->default_sacctmgr_path_version;
+		$obj->_set_sacctmgr_path(@tmp);
+	}
 
 	return;
 }
@@ -118,6 +178,159 @@ sub _init($)
 
 }
 
+#-------------------------------------------------------------------
+#	Sacctmgr/slurm versioning stuff
+#-------------------------------------------------------------------
+
+sub default_sacctmgr_path_version($)
+#This returns the default path to sacctmgr, and the default Slurm
+#version, for use when defaulting sacctmgr value during construction
+#of an instance.  The path and version are returned as elements
+#of a 2 value list.
+#
+#This version just returns the lexicals
+#	$DEFAULT_SACCTMGR_CMD and
+#	$DEFAULT_SACCTMGR_VERSION
+#defined at the top of this file.  For most cases, it is easiest to
+#just change those variables.  This routine is provided as a hook
+#in case system admins need to do something more complicated when
+#defaulting these.
+{	my $class = shift;
+
+	return ( $DEFAULT_SACCTMGR_CMD, 
+		$DEFAULT_SACCTMGR_VERSION );
+}
+
+sub sacctmgr_capabilities_by_version($$)
+#Return the appropriate sacctmgr_cmd_capabilities hash for a given
+#Slurm version number.  This is our "guess" of what various versions
+#can/cannot do.
+#Returns hash ref.
+#This is a class method
+{	my $class = shift;
+	my $slurm_version = shift;
+	my $me = __PACKAGE__ . '::sacctmgr_capabilities_by_version';
+
+	return unless $slurm_version;
+	#Strip leading/trailing whitespace from version
+	$slurm_version=~ s/^\s*//; $slurm_version =~ s/\s*$//;
+
+	my @vcomps = split /\./, $slurm_version;
+	my $slurm_major = $vcomps[0];
+
+	#REturn unknown caps if bad version given
+	return unless $slurm_major =~ /^\d+$/;
+
+	#Guess at capabilities
+	if ( $slurm_major < 15 )
+	{	#Looks like an older, pre-TRES version of slurm
+		return
+			{	trackable_resources => 0,
+			};
+	} else
+	{	#Looks like a newer version of Slurm, with TRES support
+		return
+			{	trackable_resources => 1,
+			};
+	}
+}
+
+sub _set_sacctmgr_path($$;$)
+#Sets sacctmgr path and version, and the capabilities hash
+{	my $obj = shift;
+	my $newpath = shift;
+	my $newversion = shift;
+	#This should only be called via new???
+	my $me = __PACKAGE__ . '::new';
+
+	unless ( $newpath )
+	{	$newpath = $DEFAULT_SACCTMGR_CMD;
+		unless ( $newversion )
+		{	#Only default if defaulting path as well
+			$newversion = $DEFAULT_SACCTMGR_VERSION;
+		}
+	}
+
+	$obj->set('sacctmgr', $newpath); #newpath should always be set
+	if ( $newversion )
+	{	$obj->set('slurm_version', $newversion);
+	} else
+	{	#No version given, so set version to undef
+		$obj->set('slurm_version', undef);
+	}
+	#Always clear capabilities_hash
+	$obj->set('_capabilities_hash', undef);
+}
+
+sub _determine_slurm_version($)
+#Calls 'sacctmgr --version' to get our slurm version
+#Sets slurm_version data method and returns version
+{	my $obj = shift;
+	my $me = __PACKAGE__ . '::_determine_slurm_version';
+
+	my @args = ( '--version' );
+	
+	my ( $err, @out) = $obj->_run_generic_sacctmgr_cmd_always(@args);
+
+	my $errstr;
+	if ( $err )
+	{	$errstr = "Exit code: $err";
+		my $output = join "\n", @out;
+		$errstr .= "\n$output" if $output;
+		croak "$me: Error running sacctmgr --version to get slurm version\n$errstr\nat ";
+	}
+
+	my @vlines = grep /^slurm/, @out;
+	unless ( scalar(@vlines) )
+	{	$errstr = join "\n", @out;
+		croak "$me: Unable to get slurm version from 'sacctmgr --version'\n" .
+			"Output was\n$errstr\nat ";
+	}
+
+	my $version = $vlines[0];
+	$version =~ s/^slurm\s*//;
+	$obj->set('slurm_version', $version);
+	return $version;
+}
+	
+sub sacctmgr_cmd_supports($$;$)
+#This checks if the current sshare command supports the named capability.
+#If $cachedonly is set, will only used cached information (i.e. will
+#NOT invoke an sacctmgr call to find out the version)
+#Returns 1 if supports, 0 if doesn't, and undef if unknown (should
+#only occur if $cachedonly is set)
+{	my $self = shift;
+	my $capname = shift;
+	my $cachedonly = shift;
+	my $me = __PACKAGE__ . '::sacctmgr_cmd_supports';
+
+	unless ( exists $SACCTMGR_CMD_CAPABILITIES_HASH{$capname} )
+	{	warn "$me: Unrecognized capability named '$capname' at ";
+		return 0; #Unrecognized => unsupported
+	}
+
+	my $capshash = $self->get('_capabilities_hash');
+	unless ( $capshash && ref($capshash) eq 'HASH' )
+	{	#No capabilities hash, do we have a slurm_version
+		my $svers = $self->slurm_version($cachedonly);
+
+		#svers should only be undef if cachedonly and need to look up
+		return unless defined $svers;
+
+		$capshash = $self->sacctmgr_capabilities_by_version($svers);
+		unless ( $capshash && ref($capshash) )
+		{	croak "$me: Unable to get capabilities hash for Slurm version $svers at ";
+		}
+		$self->set('_capabilities_hash', $capshash);
+	}
+	
+	my $tmp = $capshash->{$capname};
+	return $tmp if defined $tmp;
+	warn "$me: Capability $capname not in capshash at ";
+	return;
+}
+	
+#
 #-------------------------------------------------------------------
 #	Basic sacctmgr commands
 #-------------------------------------------------------------------
@@ -188,7 +401,19 @@ sub _run_generic_sacctmgr_cmd_always($@)
 	{	#Verbose mode:
 		#Output command before executing it
 		my @tmp = ($obj->sacctmgr, @args);
-		my $tmpcmd = join ' ', @tmp;
+		my $tmpsp = '';
+		my $tmpcmd = '';
+		#Quote any args with whitespace so verbose output looks better
+		foreach (@tmp)
+		{	my $tmp = $_;
+			if ( $tmp =~ /\s/ )
+			{	#Quote it if it has whitespace
+				$tmp = "'$tmp'";
+			}
+			$tmpcmd .= $tmpsp . $tmp;
+			$tmpsp = " ";
+		}
+		#my $tmpcmd = join ' ', @tmp;
 		print STDERR "[VERBOSE] $tmpcmd\n";
 	}
 	my ($err, @out ) = $obj->_noshell_backticks($mode, $cmd, @args);
@@ -206,7 +431,19 @@ sub run_generic_sacctmgr_cmd($@)
 
 	if ( $obj->dryrun )
 	{	unshift @args, $obj->sacctmgr;
-		my $cmd = join ' ', @args;
+		#my $cmd = join ' ', @args;
+		#Quote any args with whitespace so debug output looks better
+		my $cmd = '';
+		my $tmpsp = '';
+		foreach (@args)
+		{	my $tmp = $_;
+			if ( $tmp =~ /\s/ )
+			{	#quote it if has whitespace
+				$tmp = "'$tmp'";
+			}
+			$cmd .= $tmpsp . $tmp;
+			$tmpsp = ' ';
+		}
 		print STDERR "[DRYRUN] $cmd\n";
 		return [];
 	}
@@ -349,7 +586,9 @@ which are:
 The path to the Slurm B<sacctmgr> command.  Can only be set at time of
 construction.  Normally defaults to just "sacctmgr", i.e. will look for it in your
 path.  Systems staff can set a different default by changing the value of the 
-constant B<$DEFAULT_SACCTMGR_CMD> at the top of this file.
+constant B<$DEFAULT_SACCTMGR_CMD> at the top of this file.  It is recommended
+that they also set B<$DEFAULT_SACCTMGR_VERSION> at the same time if specifying
+the B<sacctmgr> command of a specific Slurm version.
 
 =item B<dryrun>:
 
@@ -383,6 +622,43 @@ take the B<sacctmgr> command, with each word as a separate argument to
 the function.  The actual B<sacctmgr> command should NOT be given, as we will
 use the one provided to the constructor.
 
+=head3 Default value for B<sacctmgr> path and Slurm version
+
+If the user does not set the path to B<sacctmgr> (and perhaps the version
+of Slurm) when constructing an instance, these will be defaulted.  It is
+recommended that system administrators modify B<Sacctmgr.pm> after installation
+to set the default path to the appropriate value for the cluster, and to
+also set the Slurm version.  However, if the system administrator did not
+modify the file, the default default is to use whatever B<sacctmgr> command
+is found in the caller's path, and the Slurm version will be determined, if needed,
+by probing the B<sacctmgr> command (which involves an extra fork and exec).
+
+For system administrators looking to default these values, there are several
+ways to proceed.  The method B<_set_defaults> handles setting all default
+values for the instance (although currently only B<sacctmgr> is defaulted).
+If B<sacctmgr> is in need of being defaulted, it calls the method
+B<default_sacctmgr_path_version> which returns the path and possibly version
+as a list.  The default version of B<default_sacctmgr_path_version> simply
+returns the lexical variables B<$DEFAULT_SACCTMGR_CMD> and 
+B<$DEFAULT_SACCTMGR_VERSION> which are defined at the top of the B<Sacctmgr.pm>
+file.  If not modified, these should be just 'sacctmgr' and undef, respectively,
+which results in whatever B<sacctmgr> in the invoker's path being executing,
+and the package not knowing what version of Slurm is being used (which means
+if the version is needed, the package will attempt to query sacctmgr to discover
+the version).
+
+In most cases, it is recommended that the system administrator just change the
+values of the lexical B<$DEFAULT_SACCTMGR_CMD> and B<$DEFAULT_SACCTMGR_VERSION> 
+at the top of the file to the full path to the sacctmgr command and the version 
+of Slurm being used on the cluster.  This will avoid issues if there are issues
+with the caller's path, and prevent extra invocations of sacctmgr to determine
+the Slurm version.
+
+If the situation is too complicated for the above, the B<default_sacctmgr_path_version>
+provides a hook wherein the system administrator can use whatever algorithm desired
+to default these values.  E.g., if you have two clusters sharing the same perl
+library but running different versions of Slurm.
+
 
 =head2 RUN-type commands
 
