@@ -13,6 +13,10 @@ my $_last_raw_output;
 sub _ebadddel_last_raw_output($)
 {	return $_last_raw_output;
 }
+sub _clear_ebadddel_last_raw_output($)
+{       $_last_raw_output = [];
+}
+
 
 sub _sacctmgr_add_cmd($)
 {	my $class = shift;
@@ -30,23 +34,37 @@ sub _sacctmgr_delete_cmd($)
 	return [ '-i', 'delete', $base ];
 }
 
-sub _my_sacctmgr_add_clause($)
+sub _sacctmgr_fields_addable($$)
+#This lists the fields we can include in a sacctmgr add/create call
+#May in general depend on version of slurm, hence sacctmgr argument.
+#Should be overloaded in all children classes
+{       my $class = shift;
+	my $sacctmgr = shift;
+        $class = ref($class) if ref($class);
+        die "Class $class forgot to overload _sacctmgr_fields_addable";
+}
+
+sub _my_sacctmgr_add_clause($$)
 #This might need to be overloaded.
 #Returns a hash ref for adding this instance into DB with sacctmgr.
 {	my $obj = shift;
+	my $sacctmgr = shift;
 	croak "Must be called as an instance method at "
 		unless $obj && ref($obj);
+	my $me = ref($obj) . '::_my_sacctmgr_add_clause';
+	die "$me: Missing req param sacctmgr at " unless $sacctmgr;
+	
 	my $namefld = $obj->_sacctmgr_name_field;
-	my $fields = $obj->_sacctmgr_fields_in_order;
+	my $fields = $obj->_sacctmgr_fields_addable($sacctmgr); 
 
 	my %hash = ();
 	FIELD: foreach my $fld (@$fields)
 	{	my $fldname=$fld;
-		$fldname = 'name' if $fld eq $namefld;
+		$fldname = $namefld if $fld eq 'name';
 		my $meth = $fld;
 		my $val = $obj->$meth;
 		next FIELD unless defined $val;
-		$hash{$fldname} = $val;
+		$hash{$fldname} = $obj->_stringify_value($val, $me);
 	}
 	return \%hash;
 }
@@ -70,15 +88,14 @@ sub sacctmgr_add($$@)
 	#Throw a sort in to make ordering deterministic for regression tests
 	KEY: foreach my $key (sort(keys %fields))
 	{	my $val = $fields{$key};
-		#This does not make sense, but I am getting a stupid
-		#error about illegal defaultqos due to extra single quotes
-		#But running the command from verbose is fine, and I do
-		#not see any extra quotes when debug print here either.
-		if ( $key eq 'defaultqos' )
-		{	push @cmd, "$key=$val";
-			next KEY;
-		}
-		push @cmd, "$key='$val'";
+		$val = $class->_stringify_value($val);
+		#push @cmd, "$key='$val'";
+		#Do not add extra quotes around '$val'; they are NOT needed
+		#(eventually being passed to execvp call, so NEVER go through
+		#shell interpolation, so not needed), and can cause issues
+		#on certain sactmgr cmds which do not strip away quotes
+		#(e.g. setting defaultqos).
+		push @cmd, "$key=$val";
 	}
 
 	my $list = $sacctmgr->run_generic_sacctmgr_cmd(@cmd);
@@ -108,7 +125,7 @@ sub sacctmgr_add_me($$@)
 	croak "No/invalid Slurm::Sacctmgr object passed to $me at "
 		unless $sacctmgr && ref($sacctmgr);
 
-	my $addclause = $obj->_my_sacctmgr_add_clause;
+	my $addclause = $obj->_my_sacctmgr_add_clause($sacctmgr);
 
 	#Add extra fields
 	$addclause = { %$addclause, %extra };
@@ -134,7 +151,10 @@ sub sacctmgr_delete($$@)
 	foreach my $key (sort (keys %where) )
 	{	my $val = $where{$key};
 		$val = '' unless defined $val;
-		push @where, "$key='$val'";
+		#push @where, "$key='$val'";
+		#Do not add extra quotes around '$val'; they are NOT needed
+		#We do NOT go through shell interpolation
+		push @where, "$key=$val";
 	}
 	if ( @where )
 	{	push @cmd, 'where', @where;
@@ -186,36 +206,23 @@ Slurm::Sacctmgr::EntityBaseAddDel
 
 This is the base class for entities managed by sacctmgr, for entities
 which can be use the "add" and "delete" commands.
-It provides the B<sacctmgr_add> and B<sacctmgr_del> methods.  The heavy liting is actually
+It provides the B<sacctmgr_add> and B<sacctmgr_del> methods.  The heavy lifting is actually
 done by B<Slurm::Sacctmgr::EntityBase>.
 
-See B<Slurm::Sacctmgr::EntityBase> for definitions of:
-
-=over 4
-
-=item B<_ro_fields> 
-
-=item B<_rw_fields>
-
-=item B<_special_fields>
-
-=item B<_required_fields>
-
-=item B<_sacctmgr_fields_in_order>
-
-=item B<_sacctmgr_entity_name>
-
-=item B<_my_sacctmgr_where_clause>
-
-=item B<_sacctmgr_name_field>
-
-=back
-
+This is a child class of B<Slurm::Sacctmgr::EntityBase>, see that class for functions
+not described here.
 
 This module defines
 
 =over 4
 
+=item B<_sacctmgr_fields_addable($sacctmgr)>
+
+This should be overloaded in any child classes.  This should return a list ref
+of field names which can be included in a B<sacctmgr add> command for this
+entity type.  As this in general may depend on the version of Slurm running,
+includes a B<Slurm::Sacctmgr> instance (which will have version of Slurm).
+
 =item B<_sacctmgr_add_cmd>
 
 The sacctmgr command to add entities of this type.  
