@@ -13,6 +13,9 @@ my $_last_raw_output;
 sub _ebmod_last_raw_output($)
 {       return $_last_raw_output;
 }
+sub _clear_ebmod_last_raw_output($)
+{	$_last_raw_output = [];
+}
 
 sub _sacctmgr_modify_cmd($)
 {	my $class = shift;
@@ -22,14 +25,26 @@ sub _sacctmgr_modify_cmd($)
 	return [ '-i', 'modify', $base ];
 }
 
-sub sacctmgr_modify($$$$)
+sub _sacctmgr_fields_updatable($$)
+#This lists the fields we can include in a sacctmgr modify call
+#May in general depend on version of slurm, hence sacctmgr argument.
+#Should be overloaded in all children classes
+{       my $class = shift;
+        my $sacctmgr = shift;
+        $class = ref($class) if ref($class);
+        die "Class $class forgot to overload _sacctmgr_fields_updatable";
+}
+
+sub sacctmgr_modify($$$$;$)
 #Does sacctmgr update one or more entities of this type, as specified
 #by $where hash ref (as key=>value pairs) to values in $update hash ref
-#(again key=>value pairs)
+#(again key=>value pairs).  Any non-string values in update hash are
+#stringified using _stringify_value.
 {	my $class = shift;
 	my $sacctmgr = shift;
 	my $where = shift || {};
 	my $update = shift;
+	my $quiet = shift;
 
 	my $me = 'sacctmgr_modify';
 	croak "No/invalid Slurm::Sacctmgr object passed to $me at "
@@ -46,8 +61,13 @@ sub sacctmgr_modify($$$$)
 	my @where = ();
 	foreach $key (sort (keys %$where))
 	{	$val = $where->{$key};
-		$val = '' unless defined $val;
-		push @where, "$key='$val'";
+		#$val = '' unless defined $val;
+		$val = $class->_stringify_value($val);
+		#push @where, "$key='$val'";
+		#No need for quoting argument, not going through shell
+		#and in some cases might cause problems with sacctmgr interpretting
+		#quotes as part of text
+		push @where, "$key=$val";
 	}
 	if (@where )
 	{	push @cmd, 'where', @where;
@@ -56,8 +76,13 @@ sub sacctmgr_modify($$$$)
 	my @set = ();
 	foreach $key (sort (keys %$update))
 	{	$val = $update->{$key};
-		$val = '' unless defined $val;
-		push @set, "$key='$val'";
+		#$val = '' unless defined $val;
+		$val = $class->_stringify_value($val);
+		#push @set, "$key='$val'";
+		#No need for quoting argument, not going through shell
+		#and in some cases might cause problems with sacctmgr interpretting
+		#quotes as part of text (e.g. setting grptresmins)
+		push @set, "$key=$val";
 	}
 	unless (@set )
 	{	croak "No fields to set for $me at ";
@@ -69,9 +94,13 @@ sub sacctmgr_modify($$$$)
 	{	croak "Error running modify cmd for $class: $list at ";
 	}
 
-	if ( scalar(@$list) )
+	if ( scalar(@$list) && ! $quiet )
 	{	my $tmp = join "\n", @$list;
-		warn "sacctmgr_modify returned:\n$tmp\n at ";
+		if ( $tmp !~ /^\s*Modified / )
+		{	#We got something other than the normal 
+			#' Modified _entity_ ...' message
+			warn "sacctmgr_modify returned:\n$tmp\n at ";
+		}
 	}
 	$_last_raw_output = $list;
 	return;
@@ -83,6 +112,7 @@ sub sacctmgr_modify_me($$@)
 {	my $obj = shift;
 	my $sacctmgr = shift;
 	my %update = @_;
+	my $quiet = delete $update{QUIET};
 
 	my $me = 'sacctmgr_modify_me';
 	croak "$me must be called as an instance method at "
@@ -91,7 +121,7 @@ sub sacctmgr_modify_me($$@)
 		unless $sacctmgr && ref($sacctmgr);
 
 	my $where = $obj->_my_sacctmgr_where_clause;
-	return $obj->sacctmgr_modify($sacctmgr, $where, \%update);
+	return $obj->sacctmgr_modify($sacctmgr, $where, \%update, $quiet);
 }
 
 1;
@@ -99,7 +129,7 @@ __END__
 
 =head1 NAME
 
-Slurm::Sacctmgr::EntityBaseAddDel
+Slurm::Sacctmgr::EntityBaseModifiable
 
 =head1 SYNOPSIS
 
@@ -122,8 +152,6 @@ See B<Slurm::Sacctmgr::EntityBase> for definitions of:
 
 =item B<_rw_fields>
 
-=item B<_special_fields>
-
 =item B<_required_fields>
 
 =item B<_sacctmgr_fields_in_order>
