@@ -26,6 +26,32 @@ sub check_results($$;$$)
 	$num_tests_run++;
 }
 
+sub stringify_value($);
+sub stringify_value($)
+#Converts refs to a string
+{	my $value = shift;
+	return '' unless defined $value;
+	return $value unless ref($value);
+
+	if ( ref($value) eq 'ARRAY' )
+	{	#Array ref, return as *ordered* csv
+		my $str = join ',', @$value;
+		return $str;
+	}
+
+	if ( ref($value) eq 'HASH' )
+	{	#Hash ref, return as csv of key=value, 
+		#alphabetized on key
+		my $str = '';
+		foreach my $key (sort (keys %$value))
+		{	my $val = $value->{$key};
+			$str .= ',' if $str;
+			$str .= "$key=$val";
+		}
+		return $str;
+	}
+}
+
 sub hash_to_arglist(@)
 #combines key=>value pairs to key=value elements
 #takes and returns lists
@@ -34,8 +60,11 @@ sub hash_to_arglist(@)
 	my @out = ();
 	my ($key, $value, $arg);
 	while ( $key = shift @in )
-	{	$value = shift @in; $value = '' unless defined $value;
-		$arg = "${key}='${value}'";
+	{	$value = shift @in;
+		$value = stringify_value($value);
+		#No quotes.  Args get called using execpe, not going through shell
+		#so we should NOT quote them
+		$arg = "${key}=${value}";
 		push @out, $arg;
 	}
 	return @out;
@@ -49,8 +78,11 @@ sub hash_to_arglist_lexical(@)
 	my @out = ();
 	my ($key, $value, $arg);
 	foreach $key (sort (keys %in) )
-	{	$value = $in{$key}; $value = '' unless defined $value;
-		$arg = "${key}='${value}'";
+	{	$value = $in{$key};
+		$value = stringify_value($value);
+		#No quotes.  Args get called using execpe, not going through shell
+		#so we should NOT quote them
+		$arg = "${key}=${value}";
 		push @out, $arg;
 	}
 	return @out;
